
use strict;
use Test::More tests => 12;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( ExistsClassByAlias LoadClassByAlias LoadRootObject );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::IniConfigAttribute';

use constant FLOAT_VALUE => 123.456;

sub Main {

    ConnectTestSite();

    # create test classes
    ok( Init(), 'initialization' );

    my $TestObject = LoadRootObject->insertChild( 'C1', { Alias => 'Test' } );
    eval { $TestObject->set({ 'NoColumn' => 1.2 }); };
    ok( ExistsError(), 'set attribute without a column' );
    is( GetError->code, 'NoResults_Attribute', 'error code NoResults_Attribute' );

    my $TestDate = DateTime->new( year => 1964, month => 10, day => 16, hour => 16, minute => 12 );
    $TestObject->set({
        A1Float => FLOAT_VALUE,
        A2Date => $TestDate->clone,
        A3Time => $TestDate->clone,
        A4DateTime => $TestDate->clone,
        A5String => 'testvalue',
    });
    is( $TestObject->get('A1Float'), FLOAT_VALUE, 'float');
    is( $TestObject->get('A2Date'), $TestDate, 'Date');
    is( $TestObject->get('A3Time'), $TestDate, 'Time');
    is( $TestObject->get('A4DateTime'), $TestDate, 'DateTime');
    is( $TestObject->get('A5String'), 'testvalue', 'string');

    $TestObject->set({
        A1Float => undef,
        A2Date => undef,
        A3Time => undef,
        A4DateTime => undef,
        A5String => undef,
    });
    is( $TestObject->get('A1Float'), undef, 'undef float');
    is( $TestObject->get('A2Date'), undef, 'undef Date');
    is( $TestObject->get('A5String'), undef, 'undef string');

    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    LoadClassByAlias( 'C1' )->delete if ExistsClassByAlias('C1');

    my $System = LoadRootObject();
    $System->child('Test')->delete if $System->existsChild('Test');
    return 1;
}

sub Init {
    Cleanup();

    my $ClassObject = LoadClassByAlias( 'Object' );

    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'A1Float', Type => 'Float', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'A2Date', Type => 'Date', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'A3Time', Type => 'Time', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'A4DateTime', Type => 'DateTime', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'A5String', Type => 'String', Package => $ATTRIBUTE_PACKAGE });

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
