/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ui.uploader
 *
 * @revision		$Revision: 1.31.2.8 $
 */

jQuery.plugin( 'de_epages.presentation.ui.uploader', {
	plugin: ['jQuery.dict', 'jQuery.tmpl','jQuery.i18n', 'jQuery.expr', 'jQuery.fn.form', 'jQuery.ui.widget', 'jQuery.ui.progressbar', 'ep.ui.dialog', 'de_epages.core', 'de_epages.presentation.ajaxTransport'],
	dict:	['ep.dict', 'de_epages.presentation.dictionary'],
	tmpl:	['de_epages.presentation.ui.uploaderProgress','de_epages.presentation.ui.uploaderError', 'de_epages.presentation.ui.uploaderReplace']
}, function( $, undefined ){

/*
 * @dictionary		ep.dict
 *
 * @translation		{Cancel}
 *					{Ignore}
 *					{Replace}
 *
 * @dictionary		de_epages.presentation.dictionary
 *
 * @translation		{FileUploadSingle}
 *					{FileUploadMultiple}
 *					{UploaderError}
 *					{UploaderReplace}
 *					{FixErrors}
 *					{FilesUploaded}
 *					{FileSizeLimitError}
 *					{InternalServerError}
 *					{MimetypeNotAcceptError}
 *					{UnknownError}
 *					{FeatureLimitExceeded}
 *					{InvalidMP3File}
 *					{InvalidImageFile}
 *					{ImageExceedsMegapixels}
 *					{VirusFoundError}
 *					{FileAlreadyExistsOnServer}
 *					{InvalidImageSize}
 */

	var epDict				= $.dict('ep.dict'),
		presentationDict	= $.dict('de_epages.presentation.dictionary'),
		tCancel				= epDict.translate('Cancel'),
		tIgnore				= epDict.translate('Ignore'),
		tReplace			= presentationDict.translate('ReplaceFiles'),
		tUploadFile			= presentationDict.translate('FileUploadSingle'),
		tUploadFiles		= presentationDict.translate('FileUploadMultiple'),
		tUploaderError		= presentationDict.translate('UploaderError'),
		tUploaderReplace	= presentationDict.translate('FileUploadMultiple'),
		tFixErrors			= presentationDict.translate('FixErrors'),
		tFilesUploaded		= presentationDict.translate('FilesUploaded'),
		tAlreadyExists    = presentationDict.translate('FileAlreadyExistsOnServer'),

		tmplHelper			= function(){
			return {
				i: 0,
				even: function(){
					return (this.i++)%2==0;
				},
				scrunch: function( string ){
					return ep.scrunch( string, 40, {ratio:0.5} );
				},
				translate: function( string ){
					var before = string.replace(/ /g,''),
						after = presentationDict.translate( before );

					return after!==before ? after : presentationDict.translate('UnknownError');
				}
			};
		};

	$.widget( "de_epages.presentation.ui.Uploader", {
		options: {
		//	multiple:	false,
		//	accept:		'*/*',
			exists:		[],
			style:		'normal',
			big:		false,
			callback:	$.noop,
			url:		'?',
			data:		{
				'ViewAction':	'JSONViewResponse'
			}
		},

		_create: function(){
			var self = this,
				o = self.options;

			self.elem = self.element
				.attr("size","1")
				.sub(ep)
				.addClass('ep-uiInput-hidden');
			// create a label as wrap
			self.wrap = ep('<label>')
				.addClass(
					'ep-uiInput-wrap de_epages-presentationUiUploader-button'
					+ ( self.elem.is(':disabled') ? ' disabled' : '' )
					+ ( self.elem.is(':readonly') ? ' readonly' : '' )
					+ ( self.elem.is(':checked') ? ' checked' : '' )
				)
				.insertBefore(self.elem)
				.append(self.elem);
			// create pseudo button
			self.button = ep('<span></span>')
				.addClass('ep-uiInput ep-uiInput-base ep-uiInput-button');

			// insert button depending on browser engine
			self.button[$.browser.msie ? 'insertAfter' : 'insertBefore'](this.elem);
			// init options
			self.option({
				'accept':	$.mime(o.accept || self.elem.attr('accept') || '*/*'),
				'multiple':	o.multiple || self.elem.attr('multiple') || false,
				'style':	o.big ? 'big' : o.style
			});

			// set wrapper to fixed width and hide overlapping content
			// for IE7, see EPG-22791
			if ($.browser.msie) {
				self.wrap
					.css({
						overflow: 'hidden',
						width: self.button.outerWidth(true) + 'px'
					});
			}

			// bind upload handle
			self.elem
				.on( 'focus.presentationUiUploader', function(){
					self.addClass("focused");
				})
				.on( 'blur.presentationUiUploader', function(){
					self.removeClass("focused");
				})
				.on( 'change.presentationUiUploader', self.Proxy('_upload') )
				.on( 'changeAttr.presentationUiUploader', self.Proxy('_changeAttr') );
			// stack of item data
			self.dataStack = [];
		},

		_setOption: function( name, value ){
			var o = this.options;

			if( value!==undefined ){
				switch( name ){
					case 'accept':
						this.elem.attr( name, $.mime(value) );
					break;
					case 'multiple':
						this.elem.attr( name, value );
						this._setOption( 'style', o.style );
					break;
					case 'big':
						this._setOption( 'style', 'big' );
					break;
					case 'style':
						switch( value ){
							case 'big':
								this.button
									.html( '<span class="ep-sprite ep-sprite-s ico_s_fileupload"></span>'+(o.multiple ? tUploadFiles : tUploadFile) )
									.addClass('ep-uiInput-big');
							break;
							case 'icon':
								this.button
									.html( '<span class="ep-sprite ep-sprite-s ico_s_fileupload ep-uiInput-buttonSpriteOnly"></span>' )
									.removeClass('ep-uiInput-big');
							break;
							case 'big-icon':
								this.button
									.html( '<span class="ep-sprite ep-sprite-m ico_m_fileupload ep-uiInput-buttonSpriteOnly"></span>' )
									.addClass('ep-uiInput-big');
							break;
							default:
								this.button
									.html( '<span class="ep-sprite ep-sprite-s ico_s_fileupload"></span>'+(o.multiple ? tUploadFiles : tUploadFile) )
									.removeClass('ep-uiInput-big');
							break;
						}
					break;
				}
			}
			return this.Inherited( arguments );
		},

		// handle addClass to the wrap
		addClass: function( name ){
			if( !(/(hover|active|focused)/).test(name) || !this.elem.is(':disabled,:readonly') ){
				this.wrap.addClass(name);
			}
		},
		// handle removeClass to the wrap
		removeClass: function( name ){
			this.wrap.removeClass(name);
		},

		// actions if change attr by javascript
		_changeAttr: function( event, attr ){
			// change disabled class
			if( attr.hasOwnProperty('disabled') ){
				if( this.elem.is(':disabled') ){
					this.addClass('disabled');
					this.removeClass('focused');
				}
				else{
					this.removeClass('disabled');
				}
			}
		},

		_upload: function( event, doReplace ){
			var self = this,
				o = self.options,
				files = self.elem[0].files,
				filesCheck = [],
				filesExists = [];

			self.statusText = '';

			self._errorClose({
				close: $.noop
			});

			for( var i=0,iLength=self.dataStack.length ; i<iLength ; i++ ){
				if( self.dataStack[i].ERROR ){
					self.dataStack.splice(i,1);
					iLength--;
					i--;
				}
			}

			// ask for replace exixting files
			if( !doReplace && o.exists.length ){
				if( !files ){
					files = [{name:self.elem.val()}];
				}

				filesCheck = $.map( files, function( file, i ){
					return file.name || file.fileName;
				});

				ep.ajax({
					async		: false,
					dataType	: 'json',
					data		: {
						ViewAction	: "JSONUnicodeConverter",
						ObjectID	: ep.config.siteId,
						Type		: "File",
						Strings		: filesCheck
					}
				})
				.done(function(data){
					filesCheck = data.Strings;
				});

				filesExists = $.map( filesCheck, function( name, i ){
					return $.inArray(name,o.exists)>=0 ? name : null;
				});

				if( filesExists.length ){
					self._uploadReplace( filesExists );
					return;
				}
			}

			self._progressOpen({
				close: function( event ){
					self._uploadCancel();
				}
			});

			// provide a flag for the server
			o.data.ReplaceExistingFiles = doReplace ? 1 : 0;

			self.uploadXHR = $.ajax({
				url:			o.url,
				data:			o.data,
				dataType:		'json',
				file:			self.elem[0],
				fileMimeAccept:	o.accept
			})
			.progress(self.Proxy('_uploadProgress'))
			.done(self.Proxy('_uploadSuccess'))
			.fail(self.Proxy('_uploadError'));
		},

		_uploadReplace: function( filesReplace ){
			this._replaceCreate();
			this.replace
				.element
				.html( $.tmpl('de_epages.presentation.ui.uploaderReplace', [{items:filesReplace, tAlreadyExists:tAlreadyExists}], tmplHelper()) );
			this._replaceOpen();
		},

		_uploadCancel: function(){
			this.uploadXHR.abort();
			this._errorClose();
		},

		_uploadProgress: function( event, statusText, jqXHR ){
			var valueItem = event.loaded * 100 / event.total,
				valueAll = event.itemLoaded * 100 / event.itemTotal;

			this.progressbarItem.progressbar( 'value', valueItem );
			this.progresstextItem.text( ep.scrunch( event.data.item.fileName, {ratio:0.5,length:65} ) );
			this.progressbarAll.progressbar( 'value', valueAll );
			this.progresstextAll.text( tFilesUploaded+': '+event.itemLoaded+' / '+event.itemTotal );

			if( /(success|error)/.test(statusText) ){
				this.dataStack.push( event.data );
			}
		},

		_uploadSuccess: function( data, statusText, jqXHR ){
			this.statusText = statusText;
			this._progressClose({
				close: $.noop
			});
			this._uploadComplete();
		},

//#JSCOVERAGE_IF 0
		_uploadError: function( jqXHR, statusText, errorThrown ){
			var self = this;
			self.statusText = statusText;
			self._errorCreate();
			self.error
				.element
				.html( $.tmpl('de_epages.presentation.ui.uploaderError', [{items: self.dataStack}], tmplHelper()) );
			self._progressClose({
				close: $.noop
			});
			self._errorOpen({
				close: function(){
					self.elem.appendTo(self.wrap);
					self._uploadComplete();
				}
			});
		},
//#JSCOVERAGE_ENDIF

		_uploadComplete: function(){
			var o			= this.options,
				dataStack	= this.dataStack,
				statusText	= this.statusText;

			this.elem.formReset();
			this.dataStack = [];
			this.statusText = '';

			if( $.type(o.callback)=='string' ){
				location.href = o.callback;
			}
			else{
				o.callback( dataStack, statusText );
			}
		},

		_progressOpen: function( options ){
			if( !this.progress ){
				this._progressCreate();
			}
			if( options ){
				this.progress.option(options);
			}
			this.progress.open();
		},

		_progressClose: function( options ){
			if( this.progress ){
				if( options ){
					this.progress.option(options);
				}
				this.progress.close();
			}
		},

		_progressCreate: function(){
			if( !this.progress ){
				var self = this;

				this.progress = ep('<div class="de_epages-presentationUiUploader">')
					.html( $.tmpl('de_epages.presentation.ui.uploaderProgress') )
					.uiDialog({
						width:		350,
						modal:		true,
						autoOpen:	false,
						title:		self.options.multiple ? tUploadFiles : tUploadFile,
						buttons:	{
							cancel:	{text:tCancel,	click:function(){ self._progressClose(); }}
						}
					})
					.find('.de_epages-presentationUiUploader-progressBar')
						.progressbar()
						.each(function(i){
							var elem = $(this);
							self[ i ? 'progressbarAll' : 'progressbarItem'] = elem;
							self[ i ? 'progresstextAll' : 'progresstextItem'] = elem.next();
						})
					.end()
					.uiDialog('Instance');
			}
		},

		_errorOpen: function( options ){
			if( !this.error ){
				this._errorCreate();
			}
			if( options ){
				this.error.option(options);
			}
			this.error.open();
		},

		_errorClose: function( options ){
			this.elem.appendTo(this.wrap);
			if( this.error ){
				if( options ){
					this.error.option(options);
				}
				this.error.close();
			}
		},

		_errorCreate: function(){
			if( !this.error ){
				var self = this;

				this.error = ep('<div class="de_epages-presentationUiUploader">')
					.html( $.tmpl('de_epages.presentation.ui.uploaderError') )
					.uiDialog({
						width:		350,
						modal:		true,
						autoOpen:	false,
						open:		function(){
							self.elem.insertAfter(self.error.options.buttons.fixit);
						},
						title:		tUploaderError,
						buttons:	{
							fixit:	{text: tFixErrors,'class':'epWidth-160'},
							ignore:	{text: tCancel,	click:function(){ self._errorClose(); }}
						}
					})
					.uiDialog('Instance');
			}
		},

		_replaceOpen: function( options ){
			if( !this.replace ){
				this._replaceCreate();
			}
			if( options ){
				this.replace.option(options);
			}

			this.replace.open();
		},

		_replaceClose: function( options ){
			if( this.replace ){
				if( options ){
					this.replace.option(options);
				}
				this.replace.close();
			}
		},

		_replaceCreate: function(){
			if( !this.replace ){
				var self = this;

				this.replace = ep('<div class="de_epages-presentationUiUploader">')
					.html( $.tmpl('de_epages.presentation.ui.uploaderError') )
					.uiDialog({
						width:		350,
						modal:		true,
						autoOpen:	false,
						title:		tUploaderReplace,
						buttons:	{
							replace:	{text: tReplace, click:function(){ self._replaceClose(); self._upload({},true); }},
							cancel:		{text: tCancel,	click:function(){ self._replaceClose(); self.statusText = "abort"; self._uploadComplete(); }}
						}
					})
					.uiDialog('Instance');
			}
		},

		destroy: function(){
			var self = this;

			if(self.error){
				ep(self.error).uiDialog('destroy');
			}
			if(self.progress){
				ep(self.progress).uiDialog('destroy');
			}

			self.elem.off('.presentationUiUploader');

			self.Inherited();
		}

	});

});
