#========================================================================================
# Test ViewRemoteSearchProducts with FakeRrequest
#
# Needs Store with DemoShop
#
#========================================================================================

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Url qw ( BuildParameterString );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::WebInterface::API::FakeRequest;
use Test::More tests => 2;

sub PostBody {
    return BuildParameterString ({
        CurrencyID   => 'EUR',
        PageSize     => 10,
        PagerSize    => 5,
        PriceFrom    => 1.0,
        PriceTo      => 200.0,
        Page         => 1,
        SearchString => 'jack',
        ViewAction   => 'RemoteSearchProducts',
    });
}

sub Tests {
    my $DemoShop = LoadObjectByPath('/Shops/DemoShop');
    plan skip_all => 'DemoShop did not initialize remotesearch'
        unless (defined $DemoShop->get('RemoteProductSearchIndex') && $DemoShop->get('RemoteProductSearchIndex')->usableAtStorefront);

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL = $DemoShop->get('WebUrl');
    my $Response = $FakeRequest->post( $URL, PostBody() );

    ok( $Response->is_success, 'ViewRemoteSearchProducts works' ) or diag( print $Response->status_line );
    like($Response->decoded_content, qr/Jack Wolfskin Blizzard Jacket/, 'checking resulting html page.');
}

RunScript(
    Debug => 1,
    Sub   => sub {
        RunOnStore(
            Store => GetTestSite,
            Sub   => sub {
                plan skip_all => 'no demoshop installed' unless (ExistsObjectByPath('/Shops/DemoShop'));
                Tests();
                done_testing();
            }
        );
    }
);
